# coding: utf-8
from collections import defaultdict
import re


def get_listeners(observer):
    u"""Возвращает слушатели, зарегистрированные в наблюдателе.

    :rtype: list
    """
    return sorted(
        observer._registered_listeners,
        key=lambda descriptor: descriptor[0]
    )


def get_listener_actions(listen_re, observer):
    u"""Возвращает действия, соответствующие регулярному выражению слушателя.

    :param str listen_re: Регулярное выражение из параметра ``listen``
        слушателя.

    :param observer: Наблюдатель objectpack.
    :type observer: objectpack.observer.base.Observer

    :rtype: dict
    """
    listen_re = re.compile(listen_re)
    result = defaultdict(list)
    for action_name, action in observer._actions.iteritems():
        if listen_re.match(action_name):
            result[action.parent].append(action)

    return result
